#include "mbed.h"
#include "TextLCD.h"

Serial pc(USBTX, USBRX); 
TextLCD lcd(p11, p12, p17, p18, p19, p20, TextLCD::LCD16x2); // rs, e, d4-d7 but could use 27,28,29,30 instead of 17,18,19,20

DigitalOut  row1(p28);                  //  row1 output
DigitalOut  row2(p27);                  //  row2 output
DigitalOut  row3(p26);                  //  row3 output
DigitalOut  led(p25);                   //  status LED (active low - low = ON - high = OFF)
DigitalIn   col1(p24);                  //  col1 input with 6k8 pullup to 3.3v
DigitalIn   col2(p23);                  //  col2 input with 6k8 pullup to 3.3v
DigitalIn   col3(p22);                  //  col3 input with 6k8 pullup to 3.3v
DigitalIn   col4(p21);                  //  col4 input with 6k8 pullup to 3.3v

int loop = 10;                          //  non zero value for the while loop
void singleled()   {
    led=0;                              //  set status led ON
    wait(0.05);                         //  wait 50ms
    led=1;                              //  set status led OFF
    wait(0.05);                         //  wait 50ms
}
void setrowslow()   {
    row1=0;                             //  set row1 low
    row2=0;                             //  set row2 low
    row3=0;                             //  set row3 low
}
void setrow1()      {
    row1=0;                             //  set row1 low
    row2=1;                             //  set row2 high
    row3=1;                             //  set row3 high
}
void setrow2()      {
    row1=1;                             //  set row1 high
    row2=0;                             //  set row2 low
    row3=1;                             //  set row3 high
}
void setrow3()      {
    row1=1;                             //  set row1 high
    row2=1;                             //  set row2 high
    row3=0;                             //  set row3 low
}
void pclf()   {
    pc.putc(0x0D);                      //  ASCII for CR
}
void key_UP()   {                       
        lcd.locate(7,1);
        lcd.printf("UP   ");
        pc.printf("UP");
        pclf();
        singleled();
}
void key_PLUS()   {
        lcd.locate(7,1);
        lcd.printf("PLUS ");
        pc.printf("PLUS");
        pclf();
        singleled();
}
void key_PREV()   {
        lcd.locate(7,1);
        lcd.printf("PREV ");
        pc.printf("PREV");
        pclf();
        singleled();
}
void key_BACK()   {
        lcd.locate(7,1);
        lcd.printf("BACK ");
        pc.printf("BACK");
        pclf();
        singleled();
}
void key_LEFT()   {
        lcd.locate(7,1);
        lcd.printf("LEFT ");
        pc.printf("LEFT");
        pclf();
        singleled();
}
void key_RIGHT()   {
        lcd.locate(7,1);
        lcd.printf("RIGHT");
        pc.printf("RIGHT");
        pclf();
        singleled();
}
void key_NEXT()   {
        lcd.locate(7,1);
        lcd.printf("NEXT ");
        pc.printf("NEXT");
        pclf();
        singleled();
}
void key_STOP()   {
        lcd.locate(7,1);
        lcd.printf("STOP ");
        pc.printf("STOP");
        pclf();
        singleled();
}
void key_DOWN()   {
        lcd.locate(7,1);
        lcd.printf("DOWN ");
        pc.printf("DOWN");
        pclf();
        singleled();
}
void key_MINUS()   {
        lcd.locate(7,1);
        lcd.printf("MINUS");
        pc.printf("MINUS");
        pclf();
        singleled();
}
void key_SPARE()   {
        lcd.locate(7,1);
        lcd.printf("SPARE");
        pc.printf("SPARE");
        pclf();
        singleled();
}
void key_OK()   {
        lcd.locate(7,1);
        lcd.printf("OK   ");
        pc.printf("OK");
        pclf();
        singleled();
}

// keyscan function routine
void keyscan() {
    setrow1();
    if (col1 == 0)
        key_UP();
    if (col2 == 0)
        key_PLUS();
    if (col3 == 0)
        key_PREV();
    if (col4 == 0)
        key_BACK();
    setrow2();
    if (col1 == 0)
        key_LEFT();
    if (col2 == 0)
        key_RIGHT();
    if (col3 == 0)
        key_NEXT();
    if (col4 == 0)
        key_STOP();
    setrow3();
    if (col1 == 0)
        key_DOWN();
    if (col2 == 0)
        key_MINUS();
    if (col3 == 0)
        key_SPARE();
    if (col4 == 0)
        key_OK();
}

//  main program routine
int main()  {
    lcd.cls();                      //  clear LCD screen
    wait(0.5);                      
    lcd.locate(4,0);
    lcd.printf("COMPANY");
    lcd.locate(1,1);
    lcd.printf("NAME");
    wait(2);
    lcd.cls();                      //  clear LCD screen
    wait(0.5);
    lcd.locate(0,0);
    lcd.printf(" Key Scan Test:");
    while (loop>0)  {
    keyscan();                      //  run keyscan routine
    wait(0.2);
    lcd.locate(7,1);                
    lcd.printf("     ");            //  blank key displayed on LCD
    }
}

